@@ -6,7 +6,7 @@
 # License           : BSD License
 # -----------------------------------------------------------------------------
 # Creation date     : 2013-JUL-15
-# Last modification : 2014-DEC-23
+# Last modification : 2014-DEC-24
 # -----------------------------------------------------------------------------
 
 import re, sys
@@ -17,7 +17,7 @@ try:
 except ImportError:
 	reporter = None
 
-VERSION = "0.0.4"
+VERSION = "0.0.5"
 LICENSE = "http://ffctn.com/doc/licenses/bsd"
 __doc__ = """
 Processor for the PythonicCSS language. This module use a PEG-based parsing
@@ -61,10 +61,10 @@ def doDedent(context):
 #
 # -----------------------------------------------------------------------------
 
-def grammar(g=Grammar("PythonicCSS")):
+def grammar(g=None):
 	"""Definition of the grammar for the PythonicCSS language, using
 	the parsing module parsing elements."""
-
+	if not g:g=Grammar("PythonicCSS", verbose=False)
 	s = g.symbols
 	g.token   ("SPACE",            "[ ]+")
 	g.token   ("TABS",             "\t*")
@@ -92,12 +92,12 @@ def grammar(g=Grammar("PythonicCSS")):
 	g.token   ("PERCENTAGE",       "\d+(\.\d+)?%")
 	g.token   ("STRING_SQ",        "'((\\\\'|[^'\\n])*)'")
 	g.token   ("STRING_DQ",        "\"((\\\\\"|[^\"\\n])*)\"")
-	g.token   ("STRING_UQ",        "[^\s\n\*\;]+")
+	g.token   ("STRING_UQ",        "[^\s\n\*\\-\+;\(\)\[\]]+")
 	g.token   ("INFIX_OPERATOR",   "[\-\+\*\/]")
 
-	g.token   ("NODE",             "\*|([a-zA-Z][a-zA-Z0-9\-]*)")
-	g.token   ("NODE_CLASS",       "\.[a-zA-Z][a-zA-Z0-9\-]*")
-	g.token   ("NODE_ID",          "#[a-zA-Z][a-zA-Z0-9\-]*")
+	g.token   ("NODE",             "\*|([a-zA-Z][\-_a-zA-Z0-9\-]*)")
+	g.token   ("NODE_CLASS",       "\.[_a-zA-Z][_a-zA-Z0-9_\-]*")
+	g.token   ("NODE_ID",          "#[_a-zA-Z][_a-zA-Z0-9\-]*")
 
 	# SEE: http://www.w3schools.com/cssref/css_units.asp
 	g.token   ("UNIT",             "em|ex|px|cm|mm|in|pt|pc|ch|rem|vh|vmin|vmax|s|deg|rad|grad|ms|Hz|kHz|\%")
@@ -146,7 +146,7 @@ def grammar(g=Grammar("PythonicCSS")):
 	g.group     ("Prefix", s.Value, g.arule(s.LP, s.Expression, s.RP))
 	s.Expression.set(s.Prefix, s.Suffixes.zeroOrMore())
 
-	g.rule      ("Invocation",   g.agroup(s.DOT,     s.METHOD_NAME).optional()._as("method"), s.LP, s.Arguments.optional()._as("arguments"), s.RP)
+	g.rule      ("Invocation",   g.arule(s.DOT,     s.METHOD_NAME).optional()._as("method"), s.LP, s.Arguments.optional()._as("arguments"), s.RP)
 	g.rule      ("InfixOperation", s.INFIX_OPERATOR, s.Expression)
 	s.Suffixes.set(s.InfixOperation, s.Invocation)
 
@@ -359,8 +359,9 @@ class Processor(AbstractProcessor):
 		elif e[0] == "I":
 			_, scope, method, args = e
 			r = ""
-			if scope:  r += scope[1]
+			if scope:  r += self._valueAsString(scope[1])
 			if method: r += "." + method
+			# TODO: Should detect the scope type and apply the corresponding method
 			r += "({0})".format(",".join((self._valueAsString(_) for _ in args or [])))
 			return (r, None)
 		else:
@@ -394,6 +395,9 @@ class Processor(AbstractProcessor):
 		else:
 			return [(r,g,b), "C"]
 
+	def onMETHOD_NAME(self, match ):
+		return match.group()
+
 	def onCOLOR_RGB(self, match ):
 		c = match.group(1).split(",")
 		if len(c) == 3:
@@ -433,7 +437,7 @@ class Processor(AbstractProcessor):
 		return [self.evaluate(_) for _ in [self.process(match[0])] + [self.process(_[1]) for _ in self.process(match[1])]]
 
 	def onInvocation( self, match, method, arguments ):
-		return ["I", None, method, arguments]
+		return ["I", None, method[1], arguments]
 
 	def onInfixOperation( self, match ):
 		op   = self.process(match[0])
@@ -445,9 +449,11 @@ class Processor(AbstractProcessor):
 
 	def onPrefix( self, match ):
 		result = self.defaultProcess(match)
-		child = match[0]
-		# FIXME: Not sure about that
-		return self.process(child)
+		child          = match[0]
+		grand_children = child.children()
+		result         = self.process(child)
+		result         = result[1] if len(result) == 3 else result
+		return result
 
 	def onExpression( self, match ):
 		prefix   = self.process(match[0])
@@ -541,7 +547,8 @@ class Processor(AbstractProcessor):
 		try:
 			evalues = [self._valueAsString(self.evaluate(_, name=name)) for _ in values]
 		except ProcessingException as e:
-			error("{0} at  offset {1}:".format(e, match.range()))
+			if reporter:
+				reporter.error("{0} at  offset {1}:".format(e, match.range()))
 			raise e
 		if name in self.PREFIXABLE_PROPERTIES:
 			res = []
@@ -671,7 +678,8 @@ class Processor(AbstractProcessor):
 
 def getGrammar():
 	global G
-	if not G: G = grammar ()
+	if not G: G = grammar()
+	# G.setVerbose(False)
 	return G
 
 def parse(path):
@@ -695,4 +703,7 @@ def run(args):
 		match = parse(a)
 		p.process(match)
 
+if __name__ == "__main__":
+	import sys
+	run(sys.argv[1:])
 # EOF
